@@ -47,7 +47,6 @@
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
 #include <kio/renamedialog.h>
-#include <kio/scheduler.h>
 #include <kparts/browseropenorsavequestion.h>
 
 // Qt
@@ -61,13 +60,23 @@
 #define QL1S(x)  QLatin1String(x)
 #define QL1C(x)  QLatin1Char(x)
 
-static bool isMimeTypeAssociatedWithSelf(const QString& mimeType, const KService::Ptr &offer)
+static void reloadRequestWithoutDisposition (QNetworkReply* reply)
+{
+    QNetworkRequest req (reply->request());
+    req.setRawHeader("x-kdewebkit-ignore-disposition", "true");
+
+    QWebFrame* frame = qobject_cast<QWebFrame*> (req.originatingObject());
+    if (!frame)
+        return;
+
+    frame->load(req);
+}
+
+static bool isMimeTypeAssociatedWithSelf(const KService::Ptr &offer)
 {
-    Q_UNUSED(mimeType);
-    
     if (!offer)
         return false;
-    
+
     kDebug(800) << offer->desktopEntryName();
 
     const QString& appName = QCoreApplication::applicationName();
@@ -134,15 +143,16 @@ static bool downloadResource (const KUrl& srcUrl, const QString& suggestedName =
 
     if (!destUrl.isValid())
         return false;
-    
-    KIO::Job *job = KIO::file_copy(srcUrl, destUrl, -1, KIO::Overwrite);
-    
+
+    KIO::Job *job = KIO::file_copy(srcUrl, destUrl);
+
     if (!metaData.isEmpty())
         job->setMetaData(metaData);
 
     job->addMetaData(QL1S("MaxCacheSize"), QL1S("0")); // Don't store in http cache.
     job->addMetaData(QL1S("cache"), QL1S("cache")); // Use entry from cache if available.
-    job->uiDelegate()->setAutoErrorHandlingEnabled(true);
+    job->ui()->setWindow((parent ? parent->window() : 0));
+    job->ui()->setAutoErrorHandlingEnabled(true);
     return true;
 }
 
@@ -170,10 +180,12 @@ public:
     KWebPagePrivate() : inPrivateBrowsingMode(false) {}
     void _k_copyResultToTempFile(KJob * job)
     {
-        if ( job->error() )
+        if ( job->error() ) {
             job->uiDelegate()->showErrorMessage();
-        else // Same as KRun::foundMimeType but with a different URL
-            (void)KRun::runUrl(static_cast<KIO::FileCopyJob *>(job)->destUrl(), mimeType, window);
+            return;
+        }
+        // Same as KRun::foundMimeType but with a different URL
+        (void)KRun::runUrl(static_cast<KIO::FileCopyJob *>(job)->destUrl(), mimeType, window);
     }
 
     QPointer<QWidget> window;
@@ -182,6 +194,19 @@ public:
     bool inPrivateBrowsingMode;
 };
 
+static void setActionIcon(QAction* action, const QIcon& icon)
+{
+    if (action) {
+        action->setIcon(icon);
+    }
+}
+
+static void setActionShortcut(QAction* action, const KShortcut& shortcut)
+{
+    if (action) {
+        action->setShortcuts(shortcut.toList());
+    }
+}
 
 KWebPage::KWebPage(QObject *parent, Integration flags)
          :QWebPage(parent), d(new KWebPagePrivate)
@@ -190,59 +215,59 @@ KWebPage::KWebPage(QObject *parent, Integration flags)
     if (!flags || (flags & KPartsIntegration))
         setPluginFactory(new KWebPluginFactory(this));
 
-    WId windowId = 0;
-    QWidget *widget = qobject_cast<QWidget*>(parent);
-    if (widget && widget->window())
-        windowId = widget->window()->winId();
+    QWidget *parentWidget = qobject_cast<QWidget*>(parent);
+    QWidget *window = parentWidget ? parentWidget->window() : 0;
 
     // KDE IO (KIO) integration...
     if (!flags || (flags & KIOIntegration)) {
         KIO::Integration::AccessManager *manager = new KIO::Integration::AccessManager(this);
         // Disable QtWebKit's internal cache to avoid duplication with the one in KIO...
         manager->setCache(0);
-        manager->setCookieJarWindowId(windowId);
+        manager->setWindow(window);
+        manager->setEmitReadyReadOnMetaDataChange(true);
         setNetworkAccessManager(manager);
     }
 
     // KWallet integration...
-    if (!flags || (flags & KWalletIntegration))
-        setWallet(new KWebWallet(0, windowId));
+    if (!flags || (flags & KWalletIntegration)) {
+        setWallet(new KWebWallet(0, (window ? window->winId() : 0) ));
+    }
 
-    action(Back)->setIcon(KIcon("go-previous"));
-    action(Forward)->setIcon(KIcon("go-next"));
-    action(Reload)->setIcon(KIcon("view-refresh"));
-    action(Stop)->setIcon(KIcon("process-stop"));
-    action(Cut)->setIcon(KIcon("edit-cut"));
-    action(Copy)->setIcon(KIcon("edit-copy"));
-    action(Paste)->setIcon(KIcon("edit-paste"));
-    action(Undo)->setIcon(KIcon("edit-undo"));
-    action(Redo)->setIcon(KIcon("edit-redo"));
-    action(InspectElement)->setIcon(KIcon("view-process-all"));
-    action(OpenLinkInNewWindow)->setIcon(KIcon("window-new"));
-    action(OpenFrameInNewWindow)->setIcon(KIcon("window-new"));
-    action(OpenImageInNewWindow)->setIcon(KIcon("window-new"));
-    action(CopyLinkToClipboard)->setIcon(KIcon("edit-copy"));
-    action(CopyImageToClipboard)->setIcon(KIcon("edit-copy"));
-    action(ToggleBold)->setIcon(KIcon("format-text-bold"));
-    action(ToggleItalic)->setIcon(KIcon("format-text-italic"));
-    action(ToggleUnderline)->setIcon(KIcon("format-text-underline"));
-    action(DownloadLinkToDisk)->setIcon(KIcon("document-save"));
-    action(DownloadImageToDisk)->setIcon(KIcon("document-save"));
+    setActionIcon(action(Back), KIcon("go-previous"));
+    setActionIcon(action(Forward), KIcon("go-next"));
+    setActionIcon(action(Reload), KIcon("view-refresh"));
+    setActionIcon(action(Stop), KIcon("process-stop"));
+    setActionIcon(action(Cut), KIcon("edit-cut"));
+    setActionIcon(action(Copy), KIcon("edit-copy"));
+    setActionIcon(action(Paste), KIcon("edit-paste"));
+    setActionIcon(action(Undo), KIcon("edit-undo"));
+    setActionIcon(action(Redo), KIcon("edit-redo"));
+    setActionIcon(action(InspectElement), KIcon("view-process-all"));
+    setActionIcon(action(OpenLinkInNewWindow), KIcon("window-new"));
+    setActionIcon(action(OpenFrameInNewWindow), KIcon("window-new"));
+    setActionIcon(action(OpenImageInNewWindow), KIcon("window-new"));
+    setActionIcon(action(CopyLinkToClipboard), KIcon("edit-copy"));
+    setActionIcon(action(CopyImageToClipboard), KIcon("edit-copy"));
+    setActionIcon(action(ToggleBold), KIcon("format-text-bold"));
+    setActionIcon(action(ToggleItalic), KIcon("format-text-italic"));
+    setActionIcon(action(ToggleUnderline), KIcon("format-text-underline"));
+    setActionIcon(action(DownloadLinkToDisk), KIcon("document-save"));
+    setActionIcon(action(DownloadImageToDisk), KIcon("document-save"));
 
     settings()->setWebGraphic(QWebSettings::MissingPluginGraphic, KIcon("preferences-plugin").pixmap(32, 32));
     settings()->setWebGraphic(QWebSettings::MissingImageGraphic, KIcon("image-missing").pixmap(32, 32));
     settings()->setWebGraphic(QWebSettings::DefaultFrameIconGraphic, KIcon("applications-internet").pixmap(32, 32));
 
-    action(Back)->setShortcut(KStandardShortcut::back().primary());
-    action(Forward)->setShortcut(KStandardShortcut::forward().primary());
-    action(Reload)->setShortcut(KStandardShortcut::reload().primary());
-    action(Stop)->setShortcut(QKeySequence(Qt::Key_Escape));
-    action(Cut)->setShortcut(KStandardShortcut::cut().primary());
-    action(Copy)->setShortcut(KStandardShortcut::copy().primary());
-    action(Paste)->setShortcut(KStandardShortcut::paste().primary());
-    action(Undo)->setShortcut(KStandardShortcut::undo().primary());
-    action(Redo)->setShortcut(KStandardShortcut::redo().primary());
-    action(SelectAll)->setShortcut(KStandardShortcut::selectAll().primary());
+    setActionShortcut(action(Back), KStandardShortcut::back());
+    setActionShortcut(action(Forward), KStandardShortcut::forward());
+    setActionShortcut(action(Reload), KStandardShortcut::reload());
+    setActionShortcut(action(Stop), KShortcut(QKeySequence(Qt::Key_Escape)));
+    setActionShortcut(action(Cut), KStandardShortcut::cut());
+    setActionShortcut(action(Copy), KStandardShortcut::copy());
+    setActionShortcut(action(Paste), KStandardShortcut::paste());
+    setActionShortcut(action(Undo), KStandardShortcut::undo());
+    setActionShortcut(action(Redo), KStandardShortcut::redo());
+    setActionShortcut(action(SelectAll), KStandardShortcut::selectAll());
 }
 
 KWebPage::~KWebPage()
@@ -296,112 +321,34 @@ void KWebPage::downloadUrl(const KUrl &url)
 void KWebPage::downloadResponse(QNetworkReply *reply)
 {
     Q_ASSERT(reply);
-    
+
     if (!reply)
         return;
 
     // Put the job on hold only for the protocols we know about (read: http).
     KIO::Integration::AccessManager::putReplyOnHold(reply);
 
-    // Reply url...
-    const KUrl requestUrl (reply->request().url());
-
-    // Get the top level window...
-    QWidget* topLevelWindow = view() ? view()->window() : 0;
-    
-    // Get suggested file name...
-    const KIO::MetaData& metaData = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
-    const QString suggestedFileName = metaData.value(QL1S("content-disposition-filename"));
-    const QString contentDispositionType = metaData.value(QL1S("content-disposition-type"));
-   
-    // Get the mime-type...
     QString mimeType;
-    extractMimeType(reply, mimeType);
-    // Convert executable text files to plain text...
-    if (KParts::BrowserRun::isTextExecutable(mimeType))
-        mimeType = QL1S("text/plain");
+    KIO::MetaData metaData;
 
-    //kDebug(800) << "Content-disposition:" << contentDispositionType << suggestedFileName;
-    //kDebug(800) << "Got unsupported content of type:" << mimeType << "URL:" << requestUrl;
-    //kDebug(800) << "Error code:" << reply->error() << reply->errorString();
+    if (handleReply(reply, &mimeType, &metaData)) {
+        return;
+    }
 
-    if (isReplyStatusOk(reply)) {
-        KParts::BrowserOpenOrSaveQuestion::Result result;
-        KParts::BrowserOpenOrSaveQuestion dlg(topLevelWindow, requestUrl, mimeType);
-        dlg.setSuggestedFileName(suggestedFileName);
-        dlg.setFeatures(KParts::BrowserOpenOrSaveQuestion::ServiceSelection);
-        result = dlg.askOpenOrSave();
+    const KUrl replyUrl (reply->url());
+    QWidget* topLevelWindow = view() ? view()->window() : 0;
 
-        switch (result) {
-        case KParts::BrowserOpenOrSaveQuestion::Open:
-            // Handle Post operations that return content...
-            if (reply->operation() == QNetworkAccessManager::PostOperation) {
-                d->mimeType = mimeType;
-                d->window = topLevelWindow;
-                QFileInfo finfo (suggestedFileName.isEmpty() ? requestUrl.fileName() : suggestedFileName);
-                KTemporaryFile tempFile;
-                tempFile.setSuffix(QL1C('.') + finfo.suffix());
-                tempFile.setAutoRemove(false);
-                tempFile.open();
-                KUrl destUrl;
-                destUrl.setPath(tempFile.fileName());
-                KIO::Job *job = KIO::file_copy(requestUrl, destUrl, 0600, KIO::Overwrite);
-                job->ui()->setWindow(topLevelWindow);
-                connect(job, SIGNAL(result(KJob *)),
-                        this, SLOT(_k_copyResultToTempFile(KJob*)));
-                return;
-            }
+    // Ask KRun to handle the response when mimetype is unknown
+    if (mimeType.isEmpty()) {
+        (void)new KRun(replyUrl, topLevelWindow, 0 , replyUrl.isLocalFile());
+        return;
+    }
 
-            // Ask before running any executables...
-            if (KParts::BrowserRun::allowExecution(mimeType, requestUrl)) {
-                KService::Ptr offer = dlg.selectedService();
-                // HACK: The check below is necessary to break an infinite
-                // recursion that occurs whenever this function is called as a result
-                // of receiving content that can be rendered by the app using this engine.
-                // For example a text/html header that containing a content-disposition
-                // header is received by the app using this class.
-                if (isMimeTypeAssociatedWithSelf(mimeType, offer)) {
-                    QNetworkRequest req (reply->request());
-                    req.setRawHeader("x-kdewebkit-ignore-disposition", "true");
-                    currentFrame()->load(req);
-                    return;
-                }
-
-                if (offer) {
-                    KUrl::List list;
-                    list.append(requestUrl);
-                    //kDebug(800) << "Suggested file name:" << suggestedFileName;
-                    if (offer->categories().contains(QL1S("KDE"), Qt::CaseInsensitive)) {
-                        KIO::Scheduler::publishSlaveOnHold();
-                        KRun::run(*offer, list, topLevelWindow , false, suggestedFileName);
-                        return;
-                    }
-                    // For non KDE applications, we launch and kill the slave-on-hold...
-                    KRun::run(*offer, list, topLevelWindow , false, suggestedFileName);
-                } else {
-                    (void)new KRun(requestUrl, topLevelWindow);
-                }
-            }
-            break;
-        case KParts::BrowserOpenOrSaveQuestion::Save:
-            // Do not attempt to download directories and local files...
-            if (mimeType == QL1S("inode/directory") || requestUrl.isLocalFile())
-                break;
-
-            downloadResource(requestUrl, suggestedFileName, topLevelWindow);
-            return;
-        case KParts::BrowserOpenOrSaveQuestion::Cancel:
-        default:
-            break;
-        }        
-    } else {
-        KService::Ptr offer = KMimeTypeTrader::self()->preferredService(mimeType);
-        if (isMimeTypeAssociatedWithSelf(mimeType, offer)) {
-            QNetworkRequest req (reply->request());
-            req.setRawHeader("x-kdewebkit-ignore-disposition", "true");
-            currentFrame()->load(req);
-            return;
-        }        
+    // Ask KRun::runUrl to handle the response when mimetype is inode/*
+    if (mimeType.startsWith(QL1S("inode/"), Qt::CaseInsensitive) &&
+        KRun::runUrl(replyUrl, mimeType, topLevelWindow, false, false,
+                     metaData.value(QL1S("content-disposition-filename")))) {
+        return;
     }
 }
 
@@ -509,4 +456,119 @@ bool KWebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &
     return QWebPage::acceptNavigationRequest(frame, request, type);
 }
 
+bool KWebPage::handleReply(QNetworkReply* reply, QString* contentType, KIO::MetaData* metaData)
+{
+    // Reply url...
+    const KUrl replyUrl (reply->url());
+
+    // Get the top level window...
+    QWidget* topLevelWindow = view() ? view()->window() : 0;
+
+    // Get suggested file name...
+    const KIO::MetaData& data = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
+    const QString suggestedFileName = data.value(QL1S("content-disposition-filename"));
+    if (metaData) {
+        *metaData = data;
+    }
+
+    // Get the mime-type...
+    QString mimeType;
+    extractMimeType(reply, mimeType);
+    if (contentType) {
+        *contentType = mimeType;
+    }
+
+    // Let the calling function deal with handling empty or inode/* mimetypes...
+    if (mimeType.isEmpty() || mimeType.startsWith(QL1S("inode/"), Qt::CaseInsensitive)) {
+        return false;
+    }
+
+    // Convert executable text files to plain text...
+    if (KParts::BrowserRun::isTextExecutable(mimeType))
+        mimeType = QL1S("text/plain");
+
+    //kDebug(800) << "Content-disposition:" << suggestedFileName;
+    //kDebug(800) << "Got unsupported content of type:" << mimeType << "URL:" << replyUrl;
+    //kDebug(800) << "Error code:" << reply->error() << reply->errorString();
+
+    if (isReplyStatusOk(reply)) {
+        KParts::BrowserOpenOrSaveQuestion::Result result;
+        KParts::BrowserOpenOrSaveQuestion dlg(topLevelWindow, replyUrl, mimeType);
+        dlg.setSuggestedFileName(suggestedFileName);
+        dlg.setFeatures(KParts::BrowserOpenOrSaveQuestion::ServiceSelection);
+        result = dlg.askOpenOrSave();
+
+        switch (result) {
+        case KParts::BrowserOpenOrSaveQuestion::Open:
+            // Handle Post operations that return content...
+            if (reply->operation() == QNetworkAccessManager::PostOperation) {
+                d->mimeType = mimeType;
+                d->window = topLevelWindow;
+                QFileInfo finfo (suggestedFileName.isEmpty() ? replyUrl.fileName() : suggestedFileName);
+                KTemporaryFile tempFile;
+                tempFile.setSuffix(QL1C('.') + finfo.suffix());
+                tempFile.setAutoRemove(false);
+                tempFile.open();
+                KUrl destUrl;
+                destUrl.setPath(tempFile.fileName());
+                KIO::Job *job = KIO::file_copy(replyUrl, destUrl, 0600, KIO::Overwrite);
+                job->ui()->setWindow(topLevelWindow);
+                job->ui()->setAutoErrorHandlingEnabled(true);
+                connect(job, SIGNAL(result(KJob *)),
+                        this, SLOT(_k_copyResultToTempFile(KJob*)));
+                return true;
+            }
+
+            // Ask before running any executables...
+            if (KParts::BrowserRun::allowExecution(mimeType, replyUrl)) {
+                KService::Ptr offer = dlg.selectedService();
+                // HACK: The check below is necessary to break an infinite
+                // recursion that occurs whenever this function is called as a result
+                // of receiving content that can be rendered by the app using this engine.
+                // For example a text/html header that containing a content-disposition
+                // header is received by the app using this class.
+                if (isMimeTypeAssociatedWithSelf(offer)) {
+                    reloadRequestWithoutDisposition(reply);
+                } else {
+                    KUrl::List list;
+                    list.append(replyUrl);
+                    bool success = false;
+                    // kDebug(800) << "Suggested file name:" << suggestedFileName;
+                    if (offer) {
+                        success = KRun::run(*offer, list, topLevelWindow , false, suggestedFileName);
+                    } else {
+                        success = KRun::displayOpenWithDialog(list, topLevelWindow, false, suggestedFileName);
+                    }
+                    // For non KIO apps and cancelled Open With dialog, remove slave on hold.
+                    if (!success || (offer && !offer->categories().contains(QL1S("KDE")))) {
+                        KIO::SimpleJob::removeOnHold(); // Remove any slave-on-hold...
+                    }
+                }
+                return true;
+            }
+            // TODO: Instead of silently failing when allowExecution fails, notify
+            // the user why the requested action cannot be fulfilled...
+            break;
+        case KParts::BrowserOpenOrSaveQuestion::Save:
+            // Do not download local files...
+            if (!replyUrl.isLocalFile()) {
+                return downloadResource(replyUrl, suggestedFileName, topLevelWindow);
+            }
+            return true;
+        case KParts::BrowserOpenOrSaveQuestion::Cancel:
+        default:
+            return true;
+        }
+    } else {
+        KService::Ptr offer = KMimeTypeTrader::self()->preferredService(mimeType);
+        if (isMimeTypeAssociatedWithSelf(offer)) {
+            reloadRequestWithoutDisposition(reply);
+            return true;
+        }
+    }
+
+    return false;
+}
+
 #include "kwebpage.moc"
+
